@@ -1,17 +1,15 @@
-use crate::error::{IntErr, Interrupt, Never};
+use crate::error::{IntErr, Interrupt};
 use crate::eval::evaluate_to_value;
 use crate::ident::Ident;
 use crate::interrupt::test_int;
 use crate::num::{Base, FormattingStyle, Number};
 use crate::scope::{GetIdentError, Scope};
 use crate::value::{ApplyMulHandling, BuiltInFunction, Value};
-use std::borrow;
 use std::sync::Arc;
 
 #[derive(Clone, Debug)]
 pub(crate) enum Expr<'a> {
-    Num(Number<'a>),
-    String(borrow::Cow<'a, str>),
+    Literal(Value<'a>),
     Ident(Ident<'a>),
     Parens(Box<Expr<'a>>),
     UnaryMinus(Box<Expr<'a>>),
@@ -23,6 +21,7 @@ pub(crate) enum Expr<'a> {
     Sub(Box<Expr<'a>>, Box<Expr<'a>>),
     Mul(Box<Expr<'a>>, Box<Expr<'a>>),
     Div(Box<Expr<'a>>, Box<Expr<'a>>),
+    Mod(Box<Expr<'a>>, Box<Expr<'a>>),
     Pow(Box<Expr<'a>>, Box<Expr<'a>>),
     // Call a function or multiply the expressions
     Apply(Box<Expr<'a>>, Box<Expr<'a>>),
@@ -38,10 +37,10 @@ pub(crate) enum Expr<'a> {
 }
 
 impl<'a> Expr<'a> {
-    pub(crate) fn format<I: Interrupt>(&self, int: &I) -> Result<String, IntErr<Never, I>> {
+    pub(crate) fn format<I: Interrupt>(&self, int: &I) -> Result<String, IntErr<String, I>> {
         Ok(match self {
-            Self::Num(n) => n.format(int)?.to_string(),
-            Self::String(s) => format!(r#""{}""#, s.as_ref()),
+            Self::Literal(Value::String(s)) => format!(r#""{}""#, s.as_ref()),
+            Self::Literal(v) => v.format_to_plain_string(0, int)?,
             Self::Ident(ident) => ident.to_string(),
             Self::Parens(x) => format!("({})", x.format(int)?),
             Self::UnaryMinus(x) => format!("(-{})", x.format(int)?),
@@ -54,6 +53,7 @@ impl<'a> Expr<'a> {
             Self::Sub(a, b) => format!("({}-{})", a.format(int)?, b.format(int)?),
             Self::Mul(a, b) => format!("({}*{})", a.format(int)?, b.format(int)?),
             Self::Div(a, b) => format!("({}/{})", a.format(int)?, b.format(int)?),
+            Self::Mod(a, b) => format!("({} mod {})", a.format(int)?, b.format(int)?),
             Self::Pow(a, b) => format!("({}^{})", a.format(int)?, b.format(int)?),
             Self::Apply(a, b) => format!("({} ({}))", a.format(int)?, b.format(int)?),
             Self::ApplyFunctionCall(a, b) | Self::ApplyMul(a, b) => {
@@ -75,14 +75,14 @@ impl<'a> Expr<'a> {
 /// returns true if rhs is '-1' or '(-1)'
 fn should_compute_inverse(rhs: &Expr<'_>) -> bool {
     if let Expr::UnaryMinus(inner) = &*rhs {
-        if let Expr::Num(n) = &**inner {
+        if let Expr::Literal(Value::Num(n)) = &**inner {
             if n.is_unitless_one() {
                 return true;
             }
         }
     } else if let Expr::Parens(inner) = &*rhs {
         if let Expr::UnaryMinus(inner2) = &**inner {
-            if let Expr::Num(n) = &**inner2 {
+            if let Expr::Literal(Value::Num(n)) = &**inner2 {
                 if n.is_unitless_one() {
                     return true;
                 }
@@ -92,6 +92,7 @@ fn should_compute_inverse(rhs: &Expr<'_>) -> bool {
     false
 }
 
+#[allow(clippy::too_many_lines)]
 pub(crate) fn evaluate<'a, I: Interrupt>(
     expr: Expr<'a>,
     scope: Option<Arc<Scope<'a>>>,
@@ -105,8 +106,7 @@ pub(crate) fn evaluate<'a, I: Interrupt>(
     }
     test_int(int)?;
     Ok(match expr {
-        Expr::<'a>::Num(n) => Value::Num(n),
-        Expr::<'a>::String(s) => Value::String(s),
+        Expr::<'a>::Literal(v) => v,
         Expr::<'a>::Ident(ident) => resolve_identifier(ident, scope, context, int)?,
         Expr::<'a>::Parens(x) => eval!(*x)?,
         Expr::<'a>::UnaryMinus(x) => eval!(*x)?.handle_num(|x| Ok(-x), Expr::UnaryMinus, scope)?,
@@ -125,7 +125,7 @@ pub(crate) fn evaluate<'a, I: Interrupt>(
         Expr::<'a>::Sub(a, b) => {
             let a = eval!(*a)?;
             match a {
-                Value::Num(a) => Value::Num(a.sub(eval!(*b)?.expect_num()?, int)?),
+                Value::Num(a) => Value::Num(Box::new(a.sub(eval!(*b)?.expect_num()?, int)?)),
                 f @ Value::BuiltInFunction(_) | f @ Value::Fn(_, _, _) => f.apply(
                     Expr::<'a>::UnaryMinus(b),
                     ApplyMulHandling::OnlyApply,
@@ -139,8 +139,8 @@ pub(crate) fn evaluate<'a, I: Interrupt>(
         Expr::<'a>::Mul(a, b) => eval!(*a)?.handle_two_nums(
             eval!(*b)?,
             |a, b| a.mul(b, int).map_err(IntErr::into_string),
-            |a| |f| Expr::Mul(f, Box::new(Expr::Num(a))),
-            |a| |f| Expr::Mul(Box::new(Expr::Num(a)), f),
+            |a| |f| Expr::Mul(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
+            |a| |f| Expr::Mul(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
             scope,
         )?,
         Expr::<'a>::Apply(a, b) | Expr::<'a>::ApplyMul(a, b) => {
@@ -155,8 +155,15 @@ pub(crate) fn evaluate<'a, I: Interrupt>(
         Expr::<'a>::Div(a, b) => eval!(*a)?.handle_two_nums(
             eval!(*b)?,
             |a, b| a.div(b, int).map_err(IntErr::into_string),
-            |a| |f| Expr::Div(f, Box::new(Expr::Num(a))),
-            |a| |f| Expr::Div(Box::new(Expr::Num(a)), f),
+            |a| |f| Expr::Div(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
+            |a| |f| Expr::Div(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
+            scope,
+        )?,
+        Expr::<'a>::Mod(a, b) => eval!(*a)?.handle_two_nums(
+            eval!(*b)?,
+            |a, b| a.modulo(b, int).map_err(IntErr::into_string),
+            |a| |f| Expr::Mod(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
+            |a| |f| Expr::Mod(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
             scope,
         )?,
         Expr::<'a>::Pow(a, b) => {
@@ -178,8 +185,8 @@ pub(crate) fn evaluate<'a, I: Interrupt>(
             lhs.handle_two_nums(
                 eval!(*b)?,
                 |a, b| a.pow(b, int),
-                |a| |f| Expr::Pow(f, Box::new(Expr::Num(a))),
-                |a| |f| Expr::Pow(Box::new(Expr::Num(a)), f),
+                |a| |f| Expr::Pow(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
+                |a| |f| Expr::Pow(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
                 scope,
             )?
         }
@@ -190,7 +197,7 @@ pub(crate) fn evaluate<'a, I: Interrupt>(
         Expr::<'a>::Fn(a, b) => Value::Fn(a, b, scope),
         Expr::<'a>::Of(a, b) => match eval!(*b)?.get_object_member(a) {
             Ok(value) => value,
-            Err(msg) => return Err(msg.to_string().into()),
+            Err(msg) => return Err(msg.into()),
         },
     })
 }
@@ -202,24 +209,26 @@ fn evaluate_add<'a, I: Interrupt>(
     int: &I,
 ) -> Result<Value<'a>, IntErr<String, I>> {
     Ok(match (a, b) {
-        (Value::Num(a), Value::Num(b)) => Value::Num(a.add(b, int)?),
+        (Value::Num(a), Value::Num(b)) => Value::Num(Box::new(a.add(*b, int)?)),
         (Value::String(a), Value::String(b)) => {
             Value::String(format!("{}{}", a.as_ref(), b.as_ref()).into())
         }
-        (Value::BuiltInFunction(f), Value::Num(a)) => {
-            f.wrap_with_expr(|f| Expr::Add(f, Box::new(Expr::Num(a))), scope)
-        }
-        (Value::Num(a), Value::BuiltInFunction(f)) => {
-            f.wrap_with_expr(|f| Expr::Add(Box::new(Expr::Num(a)), f), scope)
-        }
+        (Value::BuiltInFunction(f), Value::Num(a)) => f.wrap_with_expr(
+            |f| Expr::Add(f, Box::new(Expr::Literal(Value::Num(a)))),
+            scope,
+        ),
+        (Value::Num(a), Value::BuiltInFunction(f)) => f.wrap_with_expr(
+            |f| Expr::Add(Box::new(Expr::Literal(Value::Num(a))), f),
+            scope,
+        ),
         (Value::Fn(param, expr, scope), Value::Num(a)) => Value::Fn(
             param,
-            Box::new(Expr::Add(expr, Box::new(Expr::Num(a)))),
+            Box::new(Expr::Add(expr, Box::new(Expr::Literal(Value::Num(a))))),
             scope,
         ),
         (Value::Num(a), Value::Fn(param, expr, scope)) => Value::Fn(
             param,
-            Box::new(Expr::Add(Box::new(Expr::Num(a)), expr)),
+            Box::new(Expr::Add(Box::new(Expr::Literal(Value::Num(a))), expr)),
             scope,
         ),
         _ => return Err("expected a number".to_string().into()),
@@ -235,12 +244,16 @@ fn evaluate_as<'a, I: Interrupt>(
 ) -> Result<Value<'a>, IntErr<String, I>> {
     if let Expr::Ident(ident) = &b {
         match ident.as_str() {
+            "bool" | "boolean" => {
+                let num = evaluate(a, scope.clone(), context, int)?.expect_num()?;
+                return Ok((!num.is_zero()).into());
+            }
             "date" => {
                 let a = evaluate(a, scope.clone(), context, int)?;
                 return if let Value::String(s) = a {
-                    Ok(Value::Date(
-                        crate::date::Date::parse(s.as_ref()).map_err(|e| e.to_string())?,
-                    ))
+                    Ok(crate::date::Date::parse(s.as_ref())
+                        .map_err(|e| e.to_string())?
+                        .into())
                 } else {
                     Err("expected a string".to_string().into())
                 };
@@ -265,7 +278,9 @@ fn evaluate_as<'a, I: Interrupt>(
                             .to_string()
                             .into());
                     }
-                    let value = Value::Num(Number::from(u64::from(ch as u32)).with_base(Base::HEX));
+                    let value = Value::Num(Box::new(
+                        Number::from(u64::from(ch as u32)).with_base(Base::HEX),
+                    ));
                     return Ok(value);
                 }
                 return Err("expected a string".to_string().into());
@@ -274,16 +289,16 @@ fn evaluate_as<'a, I: Interrupt>(
         }
     }
     Ok(match evaluate(b, scope.clone(), context, int)? {
-        Value::Num(b) => Value::Num(
+        Value::Num(b) => Value::Num(Box::new(
             evaluate(a, scope, context, int)?
                 .expect_num()?
-                .convert_to(b, int)?,
-        ),
-        Value::Format(fmt) => Value::Num(
+                .convert_to(*b, int)?,
+        )),
+        Value::Format(fmt) => Value::Num(Box::new(
             evaluate(a, scope, context, int)?
                 .expect_num()?
                 .with_format(fmt),
-        ),
+        )),
         Value::Dp => {
             return Err(
                 "you need to specify what number of decimal places to use, e.g. '10 dp'"
@@ -298,11 +313,11 @@ fn evaluate_as<'a, I: Interrupt>(
                     .into(),
             );
         }
-        Value::Base(base) => Value::Num(
+        Value::Base(base) => Value::Num(Box::new(
             evaluate(a, scope, context, int)?
                 .expect_num()?
                 .with_base(base),
-        ),
+        )),
         Value::BuiltInFunction(_) | Value::Fn(_, _, _) => {
             return Err("unable to convert value to a function".to_string().into());
         }
@@ -312,8 +327,8 @@ fn evaluate_as<'a, I: Interrupt>(
         Value::String(_) => {
             return Err("cannot convert value to string".to_string().into());
         }
-        Value::Date(_) => {
-            return Err("cannot convert value to date".to_string().into());
+        Value::Dynamic(d) => {
+            return Err(format!("cannot convert value to {}", d.type_name()).into());
         }
     })
 }
@@ -340,13 +355,16 @@ pub(crate) fn resolve_identifier<'a, I: Interrupt>(
         }
     }
     Ok(match ident.as_str() {
-        "pi" | "\u{3c0}" => Value::Num(Number::pi()),
-        "tau" | "\u{3c4}" => Value::Num(Number::pi().mul(2.into(), int)?),
+        "pi" | "\u{3c0}" => Value::Num(Box::new(Number::pi())),
+        "tau" | "\u{3c4}" => Value::Num(Box::new(Number::pi().mul(2.into(), int)?)),
         "e" => evaluate_to_value("approx. 2.718281828459045235", scope, context, int)?,
-        "i" => Value::Num(Number::i()),
+        "phi" => evaluate_to_value("(1 + sqrt(5))/2", scope, context, int)?,
+        "i" => Value::Num(Box::new(Number::i())),
+        "true" => Value::from(true),
+        "false" => Value::from(false),
         "sqrt" => evaluate_to_value("x: x^(1/2)", scope, context, int)?,
         "cbrt" => evaluate_to_value("x: x^(1/3)", scope, context, int)?,
-        "conjugate" => Value::BuiltInFunction(BuiltInFunction::Conjugate),
+        "conjugate" => crate::value::func::CONJUGATE.into(),
         "abs" => Value::BuiltInFunction(BuiltInFunction::Abs),
         "sin" => Value::BuiltInFunction(BuiltInFunction::Sin),
         "cos" => Value::BuiltInFunction(BuiltInFunction::Cos),
@@ -364,6 +382,7 @@ pub(crate) fn resolve_identifier<'a, I: Interrupt>(
         "ln" => Value::BuiltInFunction(BuiltInFunction::Ln),
         "log2" => Value::BuiltInFunction(BuiltInFunction::Log2),
         "log" | "log10" => Value::BuiltInFunction(BuiltInFunction::Log10),
+        "not" => crate::value::func::NOT.into(),
         "exp" => evaluate_to_value("x: e^x", scope, context, int)?,
         "approx." | "approximately" => Value::BuiltInFunction(BuiltInFunction::Approximately),
         "auto" => Value::Format(FormattingStyle::Auto),
@@ -390,17 +409,17 @@ pub(crate) fn resolve_identifier<'a, I: Interrupt>(
             ("volume", eval_box!("1.08321e12 km^3")),
         ]),
         "differentiate" => Value::BuiltInFunction(BuiltInFunction::Differentiate),
-        "today" => Value::Date(crate::date::Date::today(context).map_err(|e| e.to_string())?),
-        "tomorrow" => Value::Date(
-            crate::date::Date::today(context)
-                .map_err(|e| e.to_string())?
-                .next(),
-        ),
-        "yesterday" => Value::Date(
-            crate::date::Date::today(context)
-                .map_err(|e| e.to_string())?
-                .prev(),
-        ),
+        "today" => crate::date::Date::today(context)
+            .map_err(|e| e.to_string())?
+            .into(),
+        "tomorrow" => crate::date::Date::today(context)
+            .map_err(|e| e.to_string())?
+            .next()
+            .into(),
+        "yesterday" => crate::date::Date::today(context)
+            .map_err(|e| e.to_string())?
+            .prev()
+            .into(),
         _ => {
             return crate::units::query_unit(ident.as_str(), context, int)
                 .map_err(IntErr::into_string)
