@@ -20,6 +20,7 @@ pub(crate) enum Symbol {
     Sub,
     Mul,
     Div,
+    Mod,
     Pow,
     ArrowConversion,
     Factorial,
@@ -122,6 +123,7 @@ impl fmt::Display for Symbol {
             Self::Sub => "-",
             Self::Mul => "*",
             Self::Div => "/",
+            Self::Mod => "mod",
             Self::Pow => "^",
             Self::ArrowConversion => "->",
             Self::Factorial => "!",
@@ -474,6 +476,7 @@ fn parse_ident(input: &str, allow_dots: bool) -> Result<(Token<'_>, &str), Error
             "to" | "as" | "in" => Token::Symbol(Symbol::ArrowConversion),
             "per" => Token::Symbol(Symbol::Div),
             "of" => Token::Symbol(Symbol::Of),
+            "mod" => Token::Symbol(Symbol::Mod),
             _ => Token::Ident(Ident::new(ident)),
         },
         input,
@@ -495,21 +498,22 @@ fn parse_symbol<'a>(ch: char, input: &mut &'a str) -> Result<Token<'a>, Error> {
         ')' => Symbol::CloseParens,
         '+' => Symbol::Add,
         '!' => Symbol::Factorial,
-        '-' => {
+        '-' | '\u{2212}' => {
+            // unicode minus sign
             if test_next('>') {
                 Symbol::ArrowConversion
             } else {
                 Symbol::Sub
             }
         }
-        '*' => {
+        '*' | '\u{d7}' | '\u{2715}' => {
             if test_next('*') {
                 Symbol::Pow
             } else {
                 Symbol::Mul
             }
         }
-        '/' => Symbol::Div,
+        '/' | '\u{f7}' | '\u{2215}' => Symbol::Div, // unicode division symbol and slash
         '^' => Symbol::Pow,
         ':' => Symbol::Fn,
         '=' => {
