@@ -3,10 +3,10 @@
 #![deny(clippy::pedantic)]
 #![deny(clippy::use_self)]
 #![forbid(clippy::needless_borrow)]
-#![forbid(clippy::cognitive_complexity)]
+//#![forbid(clippy::cognitive_complexity)]
 #![forbid(unreachable_pub)]
 #![forbid(elided_lifetimes_in_paths)]
-#![doc(html_root_url = "https://docs.rs/fend-core/0.1.15")]
+#![doc(html_root_url = "https://docs.rs/fend-core/0.1.17")]
 
 mod ast;
 mod date;
@@ -41,6 +41,7 @@ pub enum SpanKind {
     Date,
     Whitespace,
     Ident,
+    Boolean,
     Other,
 }
 
@@ -196,7 +197,7 @@ pub fn evaluate_with_interrupt(
 }
 
 const fn get_version_as_str() -> &'static str {
-    "0.1.15"
+    "0.1.17"
 }
 
 /// Returns the current version of `fend-core`.
