@@ -6,9 +6,9 @@ use crate::scope::Scope;
 use crate::{ast, ident::Ident};
 use crate::{Span, SpanKind};
 use std::collections::HashMap;
-use std::fmt;
 use std::ops::Neg;
 use std::sync::Arc;
+use std::{convert, fmt};
 
 use super::Exact;
 
@@ -192,6 +192,39 @@ impl<'a> Value<'a> {
         })
     }
 
+    pub(crate) fn modulo<I: Interrupt>(
+        self,
+        rhs: Self,
+        int: &I,
+    ) -> Result<Self, IntErr<String, I>> {
+        if !self.is_unitless() || !rhs.is_unitless() {
+            return Err("modulo is only supported for only unitless numbers"
+                .to_string()
+                .into());
+        }
+        let base = self.base;
+        let format = self.format;
+        let simplifiable = self.simplifiable;
+        let lhs = self.try_as_usize(int).map_err(IntErr::into_string)?;
+        let rhs = rhs.try_as_usize(int).map_err(IntErr::into_string)?;
+        if rhs == 0 {
+            return Err("modulo by zero".to_string().into());
+        }
+        let val = lhs % rhs;
+        let val = match <u64 as convert::TryFrom<usize>>::try_from(val) {
+            Ok(val) => Self::from(val),
+            Err(_) => return Err(format!("value {} is larger than {}", val, u64::MAX).into()),
+        };
+        Ok(Self {
+            value: val.value,
+            unit: val.unit,
+            exact: val.exact,
+            base,
+            format,
+            simplifiable,
+        })
+    }
+
     fn is_unitless(&self) -> bool {
         // todo this is broken for unitless components
         self.unit.components.is_empty()
@@ -460,61 +493,19 @@ impl<'a> Value<'a> {
             self.value
                 .format(self.exact, self.format, self.base, use_parentheses, int)?;
         let mut exact = formatted_value.exact;
-        let mut unit_string = String::new();
-        if !self.unit.components.is_empty() {
-            // Pluralisation:
-            // All units should be singular, except for the last unit
-            // that has a positive exponent, iff the number is not equal to 1
-            let mut positive_components = vec![];
-            let mut negative_components = vec![];
-            let mut first = true;
-            for unit_exponent in &self.unit.components {
-                if unit_exponent.exponent < 0.into() {
-                    negative_components.push(unit_exponent);
-                } else {
-                    positive_components.push(unit_exponent);
-                }
-            }
-            let invert_negative_component =
-                !positive_components.is_empty() && negative_components.len() == 1;
-            let mut merged_components = vec![];
-            let pluralised_idx = if positive_components.is_empty() {
-                usize::MAX
-            } else {
-                positive_components.len() - 1
-            };
-            for pos_comp in positive_components {
-                merged_components.push((pos_comp, false));
-            }
-            for neg_comp in negative_components {
-                merged_components.push((neg_comp, invert_negative_component));
-            }
-            let last_component_plural = self.value != 1.into();
-            for (i, (unit_exponent, invert)) in merged_components.into_iter().enumerate() {
-                if !first || unit_exponent.unit.print_with_space() {
-                    unit_string.push(' ');
-                }
-                first = false;
-                if invert {
-                    unit_string.push('/');
-                    unit_string.push(' ');
-                }
-                let plural = last_component_plural && i == pluralised_idx;
-                let exp_format = if self.format == FormattingStyle::Auto {
-                    FormattingStyle::Exact
-                } else {
-                    self.format
-                };
-                let formatted_exp =
-                    unit_exponent.format(self.base, exp_format, plural, invert, int)?;
-                unit_string.push_str(formatted_exp.value.to_string().as_str());
-                exact = exact && formatted_exp.exact;
-            }
-        }
+        let unit_string = self.unit.format(
+            "",
+            self.value == 1.into(),
+            self.base,
+            self.format,
+            true,
+            int,
+        )?;
+        exact = exact && unit_string.exact;
         Ok(FormattedValue {
             number: formatted_value.value,
             exact,
-            unit_str: unit_string,
+            unit_str: unit_string.value,
         })
     }
 
@@ -829,13 +820,102 @@ impl<'a> Unit<'a> {
                 scale_2: scale_b.mul(&adj_b, int)?,
             })
         } else {
-            Err("units are incompatible".to_string().into())
+            let from_formatted = from
+                .format(
+                    "unitless",
+                    false,
+                    Base::default(),
+                    FormattingStyle::Auto,
+                    false,
+                    int,
+                )?
+                .value;
+            let into_formatted = into
+                .format(
+                    "unitless",
+                    false,
+                    Base::default(),
+                    FormattingStyle::Auto,
+                    false,
+                    int,
+                )?
+                .value;
+            Err(format!(
+                "cannot convert from {} to {}: units are incompatible",
+                from_formatted, into_formatted
+            )
+            .into())
         }
     }
 
     const fn unitless() -> Self {
         Self { components: vec![] }
     }
+
+    fn format<I: Interrupt>(
+        &self,
+        unitless: &str,
+        value_is_one: bool,
+        base: Base,
+        format: FormattingStyle,
+        consider_printing_space: bool,
+        int: &I,
+    ) -> Result<Exact<String>, IntErr<Never, I>> {
+        let mut unit_string = String::new();
+        if self.components.is_empty() {
+            unit_string.push_str(unitless);
+            return Ok(Exact::new(unit_string, true));
+        }
+        // Pluralisation:
+        // All units should be singular, except for the last unit
+        // that has a positive exponent, iff the number is not equal to 1
+        let mut exact = true;
+        let mut positive_components = vec![];
+        let mut negative_components = vec![];
+        let mut first = true;
+        for unit_exponent in &self.components {
+            if unit_exponent.exponent < 0.into() {
+                negative_components.push(unit_exponent);
+            } else {
+                positive_components.push(unit_exponent);
+            }
+        }
+        let invert_negative_component =
+            !positive_components.is_empty() && negative_components.len() == 1;
+        let mut merged_components = vec![];
+        let pluralised_idx = if positive_components.is_empty() {
+            usize::MAX
+        } else {
+            positive_components.len() - 1
+        };
+        for pos_comp in positive_components {
+            merged_components.push((pos_comp, false));
+        }
+        for neg_comp in negative_components {
+            merged_components.push((neg_comp, invert_negative_component));
+        }
+        let last_component_plural = !value_is_one;
+        for (i, (unit_exponent, invert)) in merged_components.into_iter().enumerate() {
+            if !first || (consider_printing_space && unit_exponent.unit.print_with_space()) {
+                unit_string.push(' ');
+            }
+            first = false;
+            if invert {
+                unit_string.push('/');
+                unit_string.push(' ');
+            }
+            let plural = last_component_plural && i == pluralised_idx;
+            let exp_format = if format == FormattingStyle::Auto {
+                FormattingStyle::Exact
+            } else {
+                format
+            };
+            let formatted_exp = unit_exponent.format(base, exp_format, plural, invert, int)?;
+            unit_string.push_str(formatted_exp.value.to_string().as_str());
+            exact = exact && formatted_exp.exact;
+        }
+        Ok(Exact::new(unit_string, true))
+    }
 }
 
 impl<'a> fmt::Debug for Unit<'a> {
