@@ -1,5 +1,6 @@
 use crate::ast::Expr;
 use crate::lexer::{Symbol, Token};
+use crate::value::Value;
 use std::fmt;
 
 pub(crate) enum ParseError {
@@ -79,7 +80,7 @@ fn parse_fixed_symbol<'a, 'b>(input: &'b [Token<'a>], symbol: Symbol) -> ParseRe
 
 fn parse_number<'a, 'b>(input: &'b [Token<'a>]) -> ParseResult<'a, 'b> {
     match parse_token(input, true)? {
-        (Token::Num(num), remaining) => Ok((Expr::Num(num), remaining)),
+        (Token::Num(num), remaining) => Ok((Expr::Literal(Value::Num(Box::new(num))), remaining)),
         _ => Err(ParseError::ExpectedANumber),
     }
 }
@@ -88,7 +89,7 @@ fn parse_ident<'a, 'b>(input: &'b [Token<'a>]) -> ParseResult<'a, 'b> {
     match parse_token(input, true)? {
         (Token::Ident(ident), remaining) => {
             if let Ok(((), remaining2)) = parse_fixed_symbol(remaining, Symbol::Of) {
-                let (inner, remaining3) = parse_ident(remaining2)?;
+                let (inner, remaining3) = parse_parens_or_literal(remaining2)?;
                 Ok((Expr::Of(ident, Box::new(inner)), remaining3))
             } else {
                 Ok((Expr::Ident(ident), remaining))
@@ -128,7 +129,7 @@ fn parse_parens_or_literal<'a, 'b>(input: &'b [Token<'a>]) -> ParseResult<'a, 'b
     match token {
         Token::Num(_) => parse_number(input),
         Token::Ident(_) => parse_ident(input),
-        Token::StringLiteral(s) => Ok((Expr::String(s), remaining)),
+        Token::StringLiteral(s) => Ok((Expr::Literal(Value::String(s)), remaining)),
         Token::Symbol(Symbol::OpenParens) => parse_parens(input),
         Token::Symbol(Symbol::Backslash) => parse_backslash_lambda(input),
         Token::Symbol(s) => Err(ParseError::UnexpectedSymbol(s)),
@@ -175,23 +176,25 @@ fn parse_apply_cont<'a, 'b>(input: &'b [Token<'a>], lhs: &Expr<'a>) -> ParseResu
     let (rhs, input) = parse_power(input, false)?;
     Ok((
         match (lhs, &rhs) {
-            (Expr::Num(_), Expr::Num(_))
-            | (Expr::UnaryMinus(_), Expr::Num(_))
-            | (Expr::ApplyMul(_, _), Expr::Num(_)) => {
+            (Expr::Literal(Value::Num(_)), Expr::Literal(Value::Num(_)))
+            | (Expr::UnaryMinus(_), Expr::Literal(Value::Num(_)))
+            | (Expr::ApplyMul(_, _), Expr::Literal(Value::Num(_))) => {
                 // this may later be parsed as a compound fraction, e.g. 1 2/3
                 // or as an addition, e.g. 6 feet 1 inch
                 return Err(ParseError::InvalidApplyOperands);
             }
-            (Expr::Num(_), Expr::Pow(a, _))
+            (Expr::Literal(Value::Num(_)), Expr::Pow(a, _))
             | (Expr::UnaryMinus(_), Expr::Pow(a, _))
             | (Expr::ApplyMul(_, _), Expr::Pow(a, _)) => {
-                if let Expr::Num(_) = **a {
+                if let Expr::Literal(Value::Num(_)) = **a {
                     return Err(ParseError::InvalidApplyOperands);
                 }
                 Expr::Apply(Box::new(lhs.clone()), Box::new(rhs))
             }
-            (_, Expr::Num(_)) => Expr::ApplyFunctionCall(Box::new(lhs.clone()), Box::new(rhs)),
-            (Expr::Num(_), _) | (Expr::ApplyMul(_, _), _) => {
+            (_, Expr::Literal(Value::Num(_))) => {
+                Expr::ApplyFunctionCall(Box::new(lhs.clone()), Box::new(rhs))
+            }
+            (Expr::Literal(Value::Num(_)), _) | (Expr::ApplyMul(_, _), _) => {
                 Expr::ApplyMul(Box::new(lhs.clone()), Box::new(rhs))
             }
             _ => Expr::Apply(Box::new(lhs.clone()), Box::new(rhs)),
@@ -202,18 +205,18 @@ fn parse_apply_cont<'a, 'b>(input: &'b [Token<'a>], lhs: &Expr<'a>) -> ParseResu
 
 fn parse_mixed_fraction<'a, 'b>(input: &'b [Token<'a>], lhs: &Expr<'a>) -> ParseResult<'a, 'b> {
     let (positive, lhs, other_factor) = match lhs {
-        Expr::Num(_) => (true, lhs, None),
+        Expr::Literal(Value::Num(_)) => (true, lhs, None),
         Expr::UnaryMinus(x) => {
-            if let Expr::Num(_) = &**x {
+            if let Expr::Literal(Value::Num(_)) = &**x {
                 (false, lhs, None)
             } else {
                 return Err(ParseError::InvalidMixedFraction);
             }
         }
         Expr::Mul(a, b) => match &**b {
-            Expr::Num(_) => (true, &**b, Some(&**a)),
+            Expr::Literal(Value::Num(_)) => (true, &**b, Some(&**a)),
             Expr::UnaryMinus(x) => {
-                if let Expr::Num(_) = &**x {
+                if let Expr::Literal(Value::Num(_)) = &**x {
                     (false, &**b, Some(&**a))
                 } else {
                     return Err(ParseError::InvalidMixedFraction);
@@ -224,13 +227,13 @@ fn parse_mixed_fraction<'a, 'b>(input: &'b [Token<'a>], lhs: &Expr<'a>) -> Parse
         _ => return Err(ParseError::InvalidMixedFraction),
     };
     let (rhs_top, input) = parse_power(input, false)?;
-    if let Expr::Num(_) = rhs_top {
+    if let Expr::Literal(Value::Num(_)) = rhs_top {
     } else {
         return Err(ParseError::InvalidMixedFraction);
     }
     let (_, input) = parse_fixed_symbol(input, Symbol::Div)?;
     let (rhs_bottom, input) = parse_power(input, false)?;
-    if let Expr::Num(_) = rhs_bottom {
+    if let Expr::Literal(Value::Num(_)) = rhs_bottom {
     } else {
         return Err(ParseError::InvalidMixedFraction);
     }
@@ -258,6 +261,12 @@ fn parse_division_cont<'a, 'b>(input: &'b [Token<'a>]) -> ParseResult<'a, 'b> {
     Ok((b, input))
 }
 
+fn parse_modulo_cont<'a, 'b>(input: &'b [Token<'a>]) -> ParseResult<'a, 'b> {
+    let (_, input) = parse_fixed_symbol(input, Symbol::Mod)?;
+    let (b, input) = parse_power(input, true)?;
+    Ok((b, input))
+}
+
 fn parse_multiplicative<'a, 'b>(input: &'b [Token<'a>]) -> ParseResult<'a, 'b> {
     let (mut res, mut input) = parse_power(input, true)?;
     loop {
@@ -267,6 +276,9 @@ fn parse_multiplicative<'a, 'b>(input: &'b [Token<'a>]) -> ParseResult<'a, 'b> {
         } else if let Ok((term, remaining)) = parse_division_cont(input) {
             res = Expr::Div(Box::new(res.clone()), Box::new(term));
             input = remaining;
+        } else if let Ok((term, remaining)) = parse_modulo_cont(input) {
+            res = Expr::Mod(Box::new(res.clone()), Box::new(term));
+            input = remaining;
         } else if let Ok((new_res, remaining)) = parse_mixed_fraction(input, &res) {
             res = new_res;
             input = remaining;
