@@ -78,6 +78,9 @@ const STANDARD_PREFIXES: &[UnitTuple] = &[
     ("zepto", "", "lp@1e-21", ""),
     ("yocto", "", "lp@1e-24", ""),
     ("k", "", "=1000", ""),
+    ("M", "", "=1,000,000", ""),
+    ("G", "", "=1,000,000,000", ""),
+    ("T", "", "=1,000,000,000,000", ""),
 ];
 
 const NON_STANDARD_PREFIXES: &[UnitTuple] = &[
@@ -100,6 +103,10 @@ const BINARY_PREFIXES: &[UnitTuple] = &[
     ("exbi", "", "lp@2^60", ""),
     ("zebi", "", "lp@2^70", ""),
     ("yobi", "", "lp@2^80", ""),
+    ("Ki", "", "=2^10", ""),
+    ("Mi", "", "=2^20", ""),
+    ("Gi", "", "=2^30", ""),
+    ("Ti", "", "=2^40", ""),
 ];
 
 const NUMBER_WORDS: &[UnitTuple] = &[
@@ -212,7 +219,7 @@ const CONSTANTS: &[UnitTuple] = &[
     ),
     ("N_A", "", "=avogadro", ""),
     (
-        "G",
+        "gravitational_constant",
         "",
         "=6.67430e-11 N m^2 / kg^2",
         "gravitational constant",
@@ -427,6 +434,7 @@ const COMMON_PHYSICAL_UNITS: &[UnitTuple] = &[
     ("Wh", "", "s@W hour", ""),
     ("bar", "", "l@1e5 Pa", "about 1 atmosphere"),
     ("diopter", "", "l@/m", "reciprocal of focal length"),
+    ("sqm", "", "=m^2", ""),
     // TODO remove these compatibility units
     ("lightyear", "lightyears", "light_year", ""),
     ("light", "", "c", ""),
@@ -440,6 +448,7 @@ const IMPERIAL_UNITS: &[UnitTuple] = &[
     ("\"", "", "inch", ""),
     ("foot", "feet", "l@12 inch", ""),
     ("ft", "", "foot", ""),
+    ("sqft", "", "=ft^2", ""),
     ("yard", "yards", "l@3 ft", ""),
     ("yd", "", "yard", ""),
     ("mile", "miles", "l@5280 ft", ""),
@@ -548,39 +557,39 @@ const CURRENCIES: &[UnitTuple] = &[
 
 const EXCHANGE_RATES: &[UnitTuple] = &[
     // retrieved from https://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml
-    // exchange rates from 2021-04-14
-    ("USD", "USD", "1.1964 _EUR", ""),
-    ("JPY", "JPY", "130.33 _EUR", ""),
-    ("BGN", "BGN", "1.9558 _EUR", ""),
-    ("CZK", "CZK", "25.929 _EUR", ""),
-    ("DKK", "DKK", "7.4372 _EUR", ""),
-    ("GBP", "GBP", "0.86918 _EUR", ""),
-    ("HUF", "HUF", "358.61 _EUR", ""),
-    ("PLN", "PLN", "4.5537 _EUR", ""),
-    ("RON", "RON", "4.9240 _EUR", ""),
-    ("SEK", "SEK", "10.1433 _EUR", ""),
-    ("CHF", "CHF", "1.1033 _EUR", ""),
-    ("ISK", "ISK", "151.70 _EUR", ""),
-    ("NOK", "NOK", "10.0745 _EUR", ""),
-    ("HRK", "HRK", "7.5703 _EUR", ""),
-    ("RUB", "RUB", "90.5504 _EUR", ""),
-    ("TRY", "TRY", "9.6792 _EUR", ""),
-    ("AUD", "AUD", "1.5561 _EUR", ""),
-    ("BRL", "BRL", "6.8189 _EUR", ""),
-    ("CAD", "CAD", "1.5026 _EUR", ""),
-    ("CNY", "CNY", "7.8146 _EUR", ""),
-    ("HKD", "HKD", "9.2915 _EUR", ""),
-    ("IDR", "IDR", "17496.57 _EUR", ""),
-    ("ILS", "ILS", "3.9311 _EUR", ""),
-    ("INR", "INR", "89.8330 _EUR", ""),
-    ("KRW", "KRW", "1333.87 _EUR", ""),
-    ("MXN", "MXN", "24.0508 _EUR", ""),
-    ("MYR", "MYR", "4.9393 _EUR", ""),
-    ("NZD", "NZD", "1.6821 _EUR", ""),
-    ("PHP", "PHP", "58.031 _EUR", ""),
-    ("SGD", "SGD", "1.5998 _EUR", ""),
-    ("THB", "THB", "37.519 _EUR", ""),
-    ("ZAR", "ZAR", "17.2989 _EUR", ""),
+    // exchange rates from 2021-06-07
+    ("USD", "USD", "(1/1.2162) _EUR", ""),
+    ("JPY", "JPY", "(1/132.98) _EUR", ""),
+    ("BGN", "BGN", "(1/1.9558) _EUR", ""),
+    ("CZK", "CZK", "(1/25.400) _EUR", ""),
+    ("DKK", "DKK", "(1/7.4369) _EUR", ""),
+    ("GBP", "GBP", "(1/0.85825) _EUR", ""),
+    ("HUF", "HUF", "(1/345.91) _EUR", ""),
+    ("PLN", "PLN", "(1/4.4662) _EUR", ""),
+    ("RON", "RON", "(1/4.9234) _EUR", ""),
+    ("SEK", "SEK", "(1/10.0552) _EUR", ""),
+    ("CHF", "CHF", "(1/1.0934) _EUR", ""),
+    ("ISK", "ISK", "(1/146.70) _EUR", ""),
+    ("NOK", "NOK", "(1/10.0483) _EUR", ""),
+    ("HRK", "HRK", "(1/7.5005) _EUR", ""),
+    ("RUB", "RUB", "(1/88.6023) _EUR", ""),
+    ("TRY", "TRY", "(1/10.4902) _EUR", ""),
+    ("AUD", "AUD", "(1/1.5678) _EUR", ""),
+    ("BRL", "BRL", "(1/6.1389) _EUR", ""),
+    ("CAD", "CAD", "(1/1.4681) _EUR", ""),
+    ("CNY", "CNY", "(1/7.7795) _EUR", ""),
+    ("HKD", "HKD", "(1/9.4355) _EUR", ""),
+    ("IDR", "IDR", "(1/17325.10) _EUR", ""),
+    ("ILS", "ILS", "(1/3.9522) _EUR", ""),
+    ("INR", "INR", "(1/88.5305) _EUR", ""),
+    ("KRW", "KRW", "(1/1351.23) _EUR", ""),
+    ("MXN", "MXN", "(1/24.0450) _EUR", ""),
+    ("MYR", "MYR", "(1/5.0205) _EUR", ""),
+    ("NZD", "NZD", "(1/1.6832) _EUR", ""),
+    ("PHP", "PHP", "(1/57.930) _EUR", ""),
+    ("SGD", "SGD", "(1/1.6093) _EUR", ""),
+    ("THB", "THB", "(1/37.933) _EUR", ""),
+    ("ZAR", "ZAR", "(1/16.3923) _EUR", ""),
 ];
 
 const ALL_UNIT_DEFS: &[&[UnitTuple]] = &[
