@@ -3,11 +3,44 @@ use crate::num::{Base, FormattingStyle, Number};
 use crate::scope::Scope;
 use crate::{ast::Expr, ident::Ident};
 use crate::{Span, SpanKind};
+use dyn_clone::DynClone;
 use std::{borrow, fmt, sync::Arc};
 
+mod boolean;
+pub(crate) mod func;
+
+pub(crate) trait ValueTrait: fmt::Debug + DynClone {
+    fn type_name(&self) -> &'static str;
+
+    fn format(&self, indent: usize, spans: &mut Vec<Span>);
+    fn get_object_member(&self, _key: &str) -> Option<Value<'static>> {
+        None
+    }
+
+    fn as_bool(&self) -> Result<bool, String> {
+        Err(format!("expected a bool (found {})", self.type_name()))
+    }
+
+    fn apply<'a>(&self, _arg: Value<'a>) -> Option<Result<Value<'a>, String>> {
+        None
+    }
+}
+
+impl<'a> Clone for Box<dyn ValueTrait + 'a> {
+    fn clone(&self) -> Self {
+        dyn_clone::clone_box(&**self)
+    }
+}
+
+impl<'a, T: ValueTrait + 'a> From<T> for Value<'a> {
+    fn from(value: T) -> Self {
+        Self::Dynamic(Box::new(value))
+    }
+}
+
 #[derive(Clone)]
 pub(crate) enum Value<'a> {
-    Num(Number<'a>),
+    Num(Box<Number<'a>>),
     BuiltInFunction(BuiltInFunction),
     Format(FormattingStyle),
     Dp,
@@ -17,7 +50,7 @@ pub(crate) enum Value<'a> {
     Fn(Ident<'a>, Box<Expr<'a>>, Option<Arc<Scope<'a>>>),
     Object(Vec<(&'a str, Box<Value<'a>>)>),
     String(borrow::Cow<'a, str>),
-    Date(crate::date::Date),
+    Dynamic(Box<dyn ValueTrait + 'a>),
 }
 
 #[derive(Copy, Clone, Eq, PartialEq, Debug)]
@@ -41,7 +74,6 @@ pub(crate) enum BuiltInFunction {
     Log10,
     Base,
     Differentiate,
-    Conjugate,
 }
 
 impl BuiltInFunction {
@@ -99,7 +131,6 @@ impl BuiltInFunction {
             Self::Log10 => "log10",
             Self::Base => "base",
             Self::Differentiate => "differentiate",
-            Self::Conjugate => "conjugate",
         }
     }
 
@@ -125,10 +156,17 @@ pub(crate) enum ApplyMulHandling {
 }
 
 impl<'a> Value<'a> {
-    pub(crate) fn expect_num<I: Interrupt>(self) -> Result<Number<'a>, IntErr<String, I>> {
+    pub(crate) fn expect_num(self) -> Result<Number<'a>, String> {
         match self {
-            Self::Num(bigrat) => Ok(bigrat),
-            _ => Err("expected a number".to_string().into()),
+            Self::Num(bigrat) => Ok(*bigrat),
+            _ => Err("expected a number".to_string()),
+        }
+    }
+
+    pub(crate) fn expect_dyn(self) -> Result<Box<dyn ValueTrait + 'a>, String> {
+        match self {
+            Self::Dynamic(d) => Ok(d),
+            _ => Err("invalid type".to_string()),
         }
     }
 
@@ -139,7 +177,7 @@ impl<'a> Value<'a> {
         scope: Option<Arc<Scope<'a>>>,
     ) -> Result<Self, IntErr<String, I>> {
         Ok(match self {
-            Self::Num(n) => Self::Num(eval_fn(n)?),
+            Self::Num(n) => Self::Num(Box::new(eval_fn(*n)?)),
             Self::Fn(param, expr, scope) => Self::Fn(param, Box::new(lazy_fn(expr)), scope),
             Self::BuiltInFunction(f) => f.wrap_with_expr(lazy_fn, scope),
             _ => return Err("expected a number".to_string().into()),
@@ -159,14 +197,14 @@ impl<'a> Value<'a> {
         scope: Option<Arc<Scope<'a>>>,
     ) -> Result<Self, IntErr<String, I>> {
         Ok(match (self, rhs) {
-            (Self::Num(a), Self::Num(b)) => Self::Num(eval_fn(a, b)?),
-            (Self::BuiltInFunction(f), Self::Num(a)) => f.wrap_with_expr(lazy_fn_lhs(a), scope),
-            (Self::Num(a), Self::BuiltInFunction(f)) => f.wrap_with_expr(lazy_fn_rhs(a), scope),
+            (Self::Num(a), Self::Num(b)) => Self::Num(Box::new(eval_fn(*a, *b)?)),
+            (Self::BuiltInFunction(f), Self::Num(a)) => f.wrap_with_expr(lazy_fn_lhs(*a), scope),
+            (Self::Num(a), Self::BuiltInFunction(f)) => f.wrap_with_expr(lazy_fn_rhs(*a), scope),
             (Self::Fn(param, expr, scope), Self::Num(a)) => {
-                Self::Fn(param, Box::new(lazy_fn_lhs(a)(expr)), scope)
+                Self::Fn(param, Box::new(lazy_fn_lhs(*a)(expr)), scope)
             }
             (Self::Num(a), Self::Fn(param, expr, scope)) => {
-                Self::Fn(param, Box::new(lazy_fn_rhs(a)(expr)), scope)
+                Self::Fn(param, Box::new(lazy_fn_rhs(*a)(expr)), scope)
             }
             _ => return Err("expected a number".to_string().into()),
         })
@@ -181,6 +219,7 @@ impl<'a> Value<'a> {
         context: &mut crate::Context,
         int: &I,
     ) -> Result<Self, IntErr<String, I>> {
+        let stringified_self = self.format_to_plain_string(0, int)?;
         Ok(match self {
             Self::Num(n) => {
                 let other = crate::ast::evaluate(other, scope.clone(), context, int)?;
@@ -214,7 +253,7 @@ impl<'a> Value<'a> {
                 let n2 = n.clone();
                 other.handle_num(
                     |x| n.mul(x, int).map_err(IntErr::into_string),
-                    |x| Expr::Mul(Box::new(Expr::Num(n2)), x),
+                    |x| Expr::Mul(Box::new(Expr::Literal(Value::Num(n2))), x),
                     scope,
                 )?
             }
@@ -226,12 +265,20 @@ impl<'a> Value<'a> {
                     Scope::with_variable(param.as_str(), other, scope.clone(), custom_scope);
                 return crate::ast::evaluate(*expr, Some(Arc::new(new_scope)), context, int);
             }
+            Self::Dynamic(d) => {
+                let other = crate::ast::evaluate(other, scope.clone(), context, int)?;
+                match d.apply(other) {
+                    None => {
+                        return Err(
+                            format!("'{}' is not a function or a number", stringified_self).into(),
+                        )
+                    }
+                    Some(Err(msg)) => return Err(msg.into()),
+                    Some(Ok(val)) => val,
+                }
+            }
             _ => {
-                return Err(format!(
-                    "'{}' is not a function or a number",
-                    self.format_to_plain_string(0, int)?
-                )
-                .into());
+                return Err(format!("'{}' is not a function or a number", stringified_self).into());
             }
         })
     }
@@ -244,7 +291,7 @@ impl<'a> Value<'a> {
         int: &I,
     ) -> Result<Self, IntErr<String, I>> {
         let arg = crate::ast::evaluate(arg, scope.clone(), context, int)?;
-        Ok(Self::Num(match func {
+        Ok(Self::Num(Box::new(match func {
             BuiltInFunction::Approximately => arg.expect_num()?.make_approximate(),
             BuiltInFunction::Abs => arg.expect_num()?.abs(int)?,
             BuiltInFunction::Sin => arg.expect_num()?.sin(scope, context, int)?,
@@ -275,8 +322,7 @@ impl<'a> Value<'a> {
                 ));
             }
             BuiltInFunction::Differentiate => return arg.differentiate("x", int),
-            BuiltInFunction::Conjugate => arg.expect_num()?.conjugate(),
-        }))
+        })))
     }
 
     pub(crate) fn format_to_plain_string<I: Interrupt>(
@@ -370,17 +416,14 @@ impl<'a> Value<'a> {
                     kind: SpanKind::String,
                 });
             }
-            Self::Date(d) => {
-                spans.push(Span {
-                    string: d.to_string(),
-                    kind: SpanKind::Date,
-                });
+            Self::Dynamic(d) => {
+                d.format(indent, spans);
             }
         }
         Ok(())
     }
 
-    pub(crate) fn get_object_member(self, key: Ident<'_>) -> Result<Self, &'static str> {
+    pub(crate) fn get_object_member(self, key: Ident<'_>) -> Result<Self, String> {
         match self {
             Self::Object(kv) => {
                 for (k, v) in kv {
@@ -388,9 +431,13 @@ impl<'a> Value<'a> {
                         return Ok(*v);
                     }
                 }
-                Err("could not find key in object")
+                Err("could not find key in object".to_string())
             }
-            _ => Err("expected an object"),
+            Self::Dynamic(d) => match d.get_object_member(key.as_str()) {
+                Some(v) => Ok(v),
+                None => Err(format!("could not find key {}", key.as_str())),
+            },
+            _ => Err("expected an object".to_string()),
         }
     }
 
@@ -400,7 +447,7 @@ impl<'a> Value<'a> {
         int: &I,
     ) -> Result<Self, IntErr<String, I>> {
         match self {
-            Self::Num(_) => Ok(Value::Num(Number::from(0))),
+            Self::Num(_) => Ok(Value::Num(Box::new(Number::from(0)))),
             Self::BuiltInFunction(f) => Ok(f
                 .differentiate()
                 .ok_or(format!("cannot differentiate built-in function {}", f))?),
@@ -437,7 +484,7 @@ impl<'a> fmt::Debug for Value<'a> {
                 write!(f, "{}", s)
             }
             Self::String(s) => write!(f, r#""{}""#, s.as_ref()),
-            Self::Date(d) => write!(f, "{:?}", d),
+            Self::Dynamic(d) => write!(f, "{:?}", d),
         }
     }
 }
