@@ -4404,7 +4404,10 @@ fn two_km_sq_to_hectares() {
 
 #[test]
 fn kg_to_unitless() {
-    expect_error("kg to unitless", Some("units are incompatible"));
+    expect_error(
+        "kg to unitless",
+        Some("cannot convert from kg to unitless: units are incompatible"),
+    );
 }
 
 #[test]
@@ -5182,3 +5185,107 @@ fn not_simplify_explicit_to() {
         "489.296375410515266426112 ms / year",
     );
 }
+
+#[test]
+fn unicode_operators() {
+    test_eval("5 − 2 ✕ 3 × 1 ÷ 1 ∕ 3", "3");
+}
+
+#[test]
+fn bool_true() {
+    test_eval("true", "true");
+}
+
+#[test]
+fn bool_false() {
+    test_eval("false", "false");
+}
+
+#[test]
+fn zero_to_bool() {
+    test_eval("0 to bool", "false");
+}
+
+#[test]
+fn zero_to_boolean() {
+    test_eval("0 to boolean", "false");
+}
+
+#[test]
+fn one_to_bool() {
+    test_eval("1 to bool", "true");
+}
+
+#[test]
+fn minus_one_to_bool() {
+    test_eval("-1 to bool", "true");
+}
+
+#[test]
+fn not_true() {
+    test_eval("not true", "false");
+}
+
+#[test]
+fn not_false() {
+    test_eval("not false", "true");
+}
+
+#[test]
+fn not_one() {
+    expect_error("not 1", None);
+}
+
+#[test]
+fn sqm() {
+    test_eval("5 sqm", "5 m^2");
+}
+
+#[test]
+fn sqft() {
+    test_eval("5 sqft", "5 ft^2");
+}
+
+#[test]
+fn modulo() {
+    for a in 0..30 {
+        for b in 1..30 {
+            let input = format!("{} mod {}", a, b);
+            let output = (a % b).to_string();
+            test_eval(&input, &output);
+        }
+    }
+}
+
+#[test]
+fn modulo_zero() {
+    expect_error("5 mod 0", Some("modulo by zero"));
+}
+
+#[test]
+fn binary_modulo() {
+    test_eval("0b1001010 mod 5", "0b100");
+}
+
+#[test]
+fn month_of_date() {
+    test_eval_simple("month of ('2020-03-04' to date)", "March");
+}
+
+#[test]
+fn weekday_of_date() {
+    test_eval_simple("day_of_week of ('2020-05-08' to date)", "Friday");
+}
+
+#[test]
+fn day_of_week_type_name() {
+    expect_error(
+        "5 to (day_of_week of ('2020-05-08' to date)",
+        Some("cannot convert value to day of week"),
+    );
+}
+
+#[test]
+fn phi() {
+    test_eval("phi", "approx. 1.6180339886");
+}
